//
// Simple wrapper for DevIL library
//

#include	"DevIL-wrap.h"
#include	<stdio.h>
#include	"libExt.h"
#include	"IL/ilut.h"

#include	<string>

using namespace std;

static	void	initDevIL ()
{
	static	bool	inited = false;
	
	if ( inited )
		return;
	
	if ( ilGetInteger ( IL_VERSION_NUM ) < IL_VERSION )
	{
		fprintf ( stderr, "Incorrect devil.dll version\n" );
		
		return;
	}
	
	if ( ilutGetInteger ( ILUT_VERSION_NUM ) < ILUT_VERSION )
	{
		fprintf ( stderr, "Incorrect ilut.dll version\n" );
		
		return;
	}
	
	ilInit       ();
	iluInit      ();
	ilutInit     ();
	ilutRenderer ( ILUT_OPENGL );
	ilEnable     ( IL_KEEP_DXTC_DATA );
	ilSetInteger ( IL_KEEP_DXTC_DATA, IL_TRUE );
	ilutEnable   ( ILUT_GL_AUTODETECT_TEXTURE_TARGET );
	ilutEnable   ( ILUT_OPENGL_CONV );
	ilutEnable   ( ILUT_GL_USE_S3TC );
	
	inited = true;
}

unsigned	loadTexture2D ( const char * fileName, bool mipmap )
{
	ILuint	texId;
	GLuint	id;
	
	initDevIL   ();
	ilGenImages ( 1, &texId );
	ilBindImage ( texId );
	
	if ( !ilLoadImage ( fileName ) )
		return 0;

	if ( !mipmap )
		id = ilutGLBindTexImage ();
	else
		id = ilutGLBindMipmaps  ();
		
	glBindTexture   ( GL_TEXTURE_2D, id );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT );
	
    if ( mipmap )
        glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,   GL_LINEAR_MIPMAP_LINEAR );
	
	glBindTexture  ( GL_TEXTURE_2D, 0 );
	ilDeleteImages ( 1, &texId );
	
	return id;
}

unsigned	loadTexture3D ( const char * fileName, bool mipmap )
{
	ILuint	texId;
	GLuint	id;
	
	initDevIL   ();
	ilGenImages ( 1, &texId );
	ilBindImage ( texId );
	
	if ( !ilLoadImage ( fileName ) )
		return 0;
		
	int	width  = ilGetInteger ( IL_IMAGE_WIDTH  );
	int	height = ilGetInteger ( IL_IMAGE_HEIGHT );
	int	depth  = ilGetInteger ( IL_IMAGE_DEPTH  );
	int type   = ilGetInteger ( IL_IMAGE_TYPE   );
	int fmt    = ilGetInteger ( IL_IMAGE_FORMAT );
	int	dxtc   = ilGetInteger ( IL_DXTC_DATA_FORMAT );
	bool comp  = (dxtc == IL_DXT1) || (dxtc == IL_DXT2) || (dxtc == IL_DXT3) || (dxtc == IL_DXT4) || (dxtc == IL_DXT5);
	
    glGenTextures ( 1, &id );
    glBindTexture ( GL_TEXTURE_3D_EXT, id );

                                                    // use SGIS_generate_mipmap extensions
    if ( mipmap )
    {
        glTexParameteri ( GL_TEXTURE_3D_EXT, GL_GENERATE_MIPMAP_SGIS, GL_TRUE );
        glTexParameteri ( GL_TEXTURE_3D_EXT, GL_TEXTURE_MIN_FILTER,   GL_LINEAR_MIPMAP_LINEAR );
    }
	
    glTexImage3DEXT ( GL_TEXTURE_3D_EXT, 0, fmt, width, height, depth, 0,
                      fmt, type, ilGetData () );
					  
    glTexParameteri ( GL_TEXTURE_3D_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_3D_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_3D_EXT, GL_TEXTURE_WRAP_S,     GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_3D_EXT, GL_TEXTURE_WRAP_T,     GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_3D_EXT, GL_TEXTURE_WRAP_R,     GL_REPEAT );

    glBindTexture ( GL_TEXTURE_3D_EXT, 0 );
	
	ilDeleteImages ( 1, &texId );

	return id;
}

unsigned	loadTextureCube ( const char * fileName, bool mipmap )		// single file -> dds
{
	ILuint	texId;
	GLuint	id;
	
	initDevIL   ();
	ilGenImages ( 1, &texId );
	ilBindImage ( texId );
	
	if ( !ilLoadImage ( fileName ) )
		return 0;

	id = ilutGLBindTexImage ();
		
	glBindTexture   ( GL_TEXTURE_CUBE_MAP_ARB, id );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S,     GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T,     GL_REPEAT );
	
	glBindTexture  ( GL_TEXTURE_CUBE_MAP_ARB, 0 );
	ilDeleteImages ( 1, &texId );
	
	return id;
}

unsigned	loadTextureCube ( const char ** files, const char * path, bool mipmap )
{
	ILuint	texId [6];
	GLuint	id;
	int		cubeSize = -1;
	int		format, type;
	int		side;
	
	initDevIL ();
	
	for ( side = 0; side < 6; side++ )
	{
		string	name = path;
		int		len  = name.length ();
		
		if ( len > 0 )
			if ( name [len-1] != '/' && name [len-1] != '\\' )
				name += '/';
				
		name += files [side];
		
		ilGenImages ( 1, &texId [side] );
		ilBindImage ( texId [side] );

		if ( !ilLoadImage ( name.c_str () ) )
			return 0;
		
		int	width  = ilGetInteger ( IL_IMAGE_WIDTH  );
		int	height = ilGetInteger ( IL_IMAGE_HEIGHT );
		int fmt    = ilGetInteger ( IL_IMAGE_FORMAT );
		
		if ( width != height )
		{
			ilDeleteImages ( 6, texId );
			
			return 0;
		}
		
		if ( cubeSize == -1 )
		{
			cubeSize = width;
			format   = fmt;
			type     = ilGetInteger ( IL_IMAGE_TYPE );
		}
			
		if ( cubeSize != width )
		{
			ilDeleteImages ( 6, texId );
			
			return 0;
		}
	}

    glGenTextures   ( 1, &id );
    glEnable        ( GL_TEXTURE_CUBE_MAP_ARB );
    glBindTexture   ( GL_TEXTURE_CUBE_MAP_ARB, id );
    glPixelStorei   ( GL_UNPACK_ALIGNMENT, 1 );   
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S,     GL_CLAMP_TO_EDGE );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T,     GL_CLAMP_TO_EDGE );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R,     GL_CLAMP_TO_EDGE );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

	GLenum	intFormat = GL_RGBA8;

	if ( format == IL_RGB || format == IL_BGR )
		intFormat = GL_RGB8;

	for ( side = 0; side < 6; side++ )
	{
		ilBindImage  ( texId [side] );
        glTexImage2D ( GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB + side, 0, intFormat,
                       cubeSize, cubeSize, 0, format, type, ilGetData () );
	}
		
    glBindTexture  ( GL_TEXTURE_CUBE_MAP_ARB, 0 );
	ilDeleteImages ( 6, texId );
	
	return id;
}
