//
// Test for loading a dds cube map via DevIL
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include	"libExt.h"
#include	"IL/ilut.h"
#include	"Vector3D.h"
#include	"Vector2D.h"
#include	"boxes.h"
#include	"DevIL-wrap.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include	<glut.h>
#endif

#include	<stdio.h>
#include	<stdlib.h>

float	 angle    = 0;
float	 angle2   = 0;
unsigned texture;
int		 reflMode = GL_REFLECTION_MAP_ARB;

void init ()
{
	glClearColor ( 0, 0, 0, 1 );
	glEnable     ( GL_DEPTH_TEST );
	glEnable     ( GL_TEXTURE_2D );

	glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
	glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void display ()
{
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glDisable       ( GL_CULL_FACE );
	glPushMatrix    ();
	glTranslatef    ( 1, 1, 1 );
	glRotatef       ( angle,  1, 1, 0 );
	glRotatef       ( angle2, 0, 1, 1 );

    glTexGeni ( GL_S, GL_TEXTURE_GEN_MODE, reflMode );
    glTexGeni ( GL_T, GL_TEXTURE_GEN_MODE, reflMode );
	glTexGeni ( GL_R, GL_TEXTURE_GEN_MODE, reflMode );

	glEnable      ( GL_TEXTURE_CUBE_MAP_ARB );
	glBindTexture ( GL_TEXTURE_CUBE_MAP_ARB, texture );

	glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri ( GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP );

    glEnable  ( GL_TEXTURE_GEN_S );
    glEnable  ( GL_TEXTURE_GEN_T );
    glEnable  ( GL_TEXTURE_GEN_R );

	glutSolidTeapot ( 0.5 );

	glPopMatrix     ();
	glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
   gluLookAt      ( 0.0, 0.0, 0.0,		// eye
					5.0, 5.0, 5.0,		// center
					0.0, 1.0, 0.0 );
					
	glMatrixMode ( GL_TEXTURE );
	glLoadIdentity  ();
	glScalef        ( 1, -1, 1 );
	glMatrixMode ( GL_MODELVIEW );				
}

void key ( unsigned char key, int x, int y )
{
	if ( key == 27 || key == 'q' || key == 'Q' )		// quit requested
		exit ( 0 );
}

void	animate ()
{
	angle  = 0.04f * glutGet ( GLUT_ELAPSED_TIME );
	angle2 = 0.01f * glutGet ( GLUT_ELAPSED_TIME );

	glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
								// initialize glut
	glutInit            ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize  ( 500, 500 );


								// create window
	glutCreateWindow ( "DevIL cubemap dds example" );

								// register handlers
	glutDisplayFunc  ( display );
	glutReshapeFunc  ( reshape );
	glutKeyboardFunc ( key     );
	glutIdleFunc     ( animate );

	if ( ilGetInteger ( IL_VERSION_NUM ) < IL_VERSION )
	{
		printf ( "Incorrect devil.dll version\n" );
		
		return 1;
	}
	
	if ( ilutGetInteger ( ILUT_VERSION_NUM ) < ILUT_VERSION )
	{
		printf ( "Incorrect ilut.dll version\n" );
		
		return 1;
	}
	
	if ( !isExtensionSupported ( "GL_ARB_texture_cube_map" ) )
	{
		printf ( "ARB_texture_cube_map NOT supported.\n" );

		return 1;
	}

	init           ();
	initExtensions ();
	texture = loadTextureCube ( "../../Textures/Cubemaps/Snow.dds", true );

	glutMainLoop ();

	return 0;
}
